<?php

class DBRes implements Iterator {
 var $arr;
  
 public function __construct() {
    $this->arr = array();
 }
 
 public function push(&$obj) {
    array_push($this->arr, $obj);
 }
 
 public function fetch($flags = 0) {
    return array_shift($this->arr);
 }

 function rewind() {
    reset($this->arr);
 }
 function current() {
    return current($this->arr);
 }
 function key() {
    return key($this->arr);
 }
 function next() {
    each($this->arr);    
 }
 function valid() {
	// We don't expect 'false' elements in array (all are arrays)
    return current($this->arr)?true:false;
 }

}

class DATABASE {
 var $dbh, $connected;
 var $driver;
 var $odbc;
 
 var $server;

 const GLOBAL_QUERY = 1;
 const SINGLE_RESULT = 2;
 
 function __construct(&$server) {
     try {
        $this->odbc = false;
	
	switch($server['driver']) {
	  case "odbc":
	    if ($server['source']) {
		$this->dbh = new PDO ("odbc:" . $server['source'], $server['user'], $server['password']);
		$this->dbname = false;
		$this->connected = true;
	    } else {
		if ($server['database']) {
		    $dbtext = ";DATABASE=" . $server['database'];
		    $this->dbname = $props['db_name'];
		    $this->connected = true;
		} else $dbtext = "";
		
		if ($server['timeout']) {
		    if ($server['ping']) {
			if ((!is_array($server['ping']))&&((!$server['host'])||(!$server['port'])))
			    throw new ADEIException(translate("The ping-before-connection feature is requiring the database's host and port to be excplicitly specified."));
			    
			$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
			if ($socket) {
			    socket_set_nonblock($socket);
			
			    if (is_array($ping))
				$res = @socket_connect($socket, $server['ping']['host'], $server['ping']['port']);
			    else
				$res = @socket_connect($socket, $server['host'], $server['port']);
			
			    if (!$res) {
				$res = socket_select($r = array($socket), $w = NULL, $e = array($socket), (int)$server['timeout']/1000000, $server['timeout']%1000000);
				if ($res == 1) $res = true;
				else $res = false;
			    }
			    socket_close($socket);
			    
			    if (!$res) {
				throw new ADEIException(translate("Error connecting to the database"));
			    }
			}
		    }
		
		    $timeout = (int)($server['timeout'] / 1000000);
		    if (!$timeout) $timeout = 1;
		    
		    $this->dbh = new PDO ("odbc:DRIVER=" . $server['subdrv'] . ";SERVER=" . $server['host'] . ";PORT=" . ($server['port']?$server['port']:"1433") . $dbtext . ";PROTOCOL=TCPIP;UID=" . $server['user'] . ";PWD=" . $server['password'], NULL, NULL, array(
			PDO::ATTR_TIMEOUT => $timeout
		    ));
		} else {
		    $this->dbh = new PDO ("odbc:DRIVER=" . $server['subdrv'] . ";SERVER=" . $server['host'] . ";PORT=" . ($server['port']?$server['port']:"1433") . $dbtext . ";PROTOCOL=TCPIP;UID=" . $server['user'] . ";PWD=" . $server['password']);
		}
	    }

	    $this->odbc = true;
	  break;
	  default:	
	    if ($server['database']) {
		$this->dbh = new PDO ($server['driver'] . ":host=" . $server['host'] . ($server['port']?(":" . $server['port']):"") . ";dbname=" . $server['database'], $server['user'], $server['password']);
		$this->dbname = $props['db_name'];	    
		$this->connected = true;
	    } else {
    		$this->dbh = new PDO ($server['driver'] . ":host=" . $server['host'] . ($server['port']?(":" . $server['port']):""), $server['user'], $server['password']);
		$this->connected = false;
	    }
	}
        
	if ($server['sqldrv']) $this->driver = $server['sqldrv'];
	else $this->driver = $server['driver'];
	
	    /* Single check for single scheme */
	switch ($this->driver) {
	    case "dblib":
		$this->driver="mssql";
	    break;
	}
    } catch(PDOException $e) {
	$this->dbh = NULL;
	$this->connected = false;

	throw new ADEIException(translate("Error connecting to the database") . ": " . $e->getMessage(), $e->getCode());
    }
    
    $this->server = $server;
 }
 
 static function GetConnectionString(array &$server, array &$options = NULL, &$application = NULL) {
    if (strtolower($server['driver']) == 'mysql') {
	if ((!$application)||($application == 'mysqldump')) {
	    return "-h" . $server['host'] . ($server['port']?(":" . $server['port']):"") . " -u" . $server['user'] . " -p" . $server['password'] . " " . $server['database'];
	} else throw new ADEIException(translate("The required application (\"%s\") is not supported at the moment", $application));
    } else throw new ADEIException(translate("The supplied connection (\"%s\") is not supported at the moment", $server['driver']));
 }
 

 function Query($sql, $flags = 0) {
//    echo $sql . "\n\n";

    if ((!$this->connected)&&(($flags&DATABASE::GLOBAL_QUERY)==0))
	 throw new ADEIException(translate("The database is not specified"));

    try {
	if ($this->odbc) {
		//	This is to prevent spurious odbc errors:
	    $stmt = $this->Prepare($sql, $flags);
	    $stmt->execute();
	
	    $resp = new DBRes();
	    if ($flags&DATABASE::SINGLE_RESULT) {
		$row = $stmt->fetch(PDO::FETCH_BOTH);
		if ($row) $resp->push($row);
	    } else {
	    	while ($row = $stmt->fetch(PDO::FETCH_BOTH)) {
		    $resp->push($row);
		}
	    }
	} else {
	    $resp = $this->dbh->query($sql);
	}
    } catch (PDOException $e) {
	throw new ADEIException(translate("SQL Query is failed with error") . ": " . $e->getMessage(), $e->getCode());
    }
//    echo "done\n";
    
    if (!$resp) {
	$e = $this->dbh->errorInfo();
	throw new ADEIException(translate("SQL Query is failed. SQL Error: %u, Driver Error: %u, Message: %s ", $e[0], $e[1], $e[2]) . "[$sql]");
    }

    return $resp;
 }

 function Prepare($sql, $flags = 0) {
    if ((!$this->connected)&&(($flags&DATABASE::GLOBAL_QUERY)==0))
	 throw new ADEIException(translate("The database is not specified"));

    try {
	$stmt = $this->dbh->prepare($sql);
    } catch (PDOException $e) {
	throw new ADEIException(translate("Can not prepare SQL Query for execution, error: ") . $e->getMessage(), $e->getCode());
    }

    if (!$stmt) {
	$e = $this->dbh->errorInfo();
	throw new ADEIException(translate("Preparation of the SQL Query is failed. SQL Error: %u, Driver Error: %u, Message: %s ", $e[0], $e[1], $e[2]) . "[$sql]");
    }

    return $stmt;
 }

 function SelectRequest($table, $columns="*", array $req = NULL) {
    $res = "SELECT ";
    if ($req['limit']) {
	switch ($this->driver) {
	    case "mssql":
		$res .= "TOP " . $req['limit'] . " ";
	    break;
	    case "mysql":
		$suffix = " LIMIT " . $req['limit'];
	    break;
	    default:
		throw new ADEIException(translate("Don't know how to handle LIMIT for '%s'", $this->driver));
	}
    }
    
    if (is_array($columns)) {
	$last = array_pop($columns);
	if (!$last) 
	    throw new ADEIException(translate("Columns list is empty"));
	
	foreach ($columns as $item) {
	    $res .=  "\"" . $item['column'] . "\", ";
	}
	$res .= "\"" . $last . "\"";
    
    } else $res .= $columns;
    
    $res .= " FROM " . $table;

    if ($req['condition']) $res .= " WHERE " . $req['condition'];
    if ($req['order']) $res .= " ORDER BY " . $req['order'];
    
    return $res . $suffix;
 }

 function GetTimeFormat() {
//    if ($this->time_format) return $this->time_format;
    
    switch ($this->driver) {
	case "mysql":
	    return "YmdHis";
	case "mssql":
	    return "Ymd H:i:s";
	    /* Hm. Didn't work from Linux
	    return "Y-d-m H:i:s";*/
	    /* We can't use '.u' since it is formated with 6 digits after the 
	    decimal point, while the MSSQL complains if there are more than 3
	    return "Y-d-m H:i:s.u";*/
	default:
	    throw new ADEIException("The date format for \"" . $this->driver . "\" is not known");
    }
 }
 
 function GetTimeRequest($column_name) {
    switch ($this->driver) {
	case "mssql":
	    return "CONVERT(CHAR(24), \"$column_name\", 21)";
	default:
	    return "\"column_name\"";
    }
 }
 
 function ShowDatabases() {
    switch ($this->driver) {
	case "mysql":
	    return $this->Query("SHOW DATABASES", DATABASE::GLOBAL_QUERY);
	case "mssql":
	    return $this->Query("SELECT name FROM master..sysdatabases", DATABASE::GLOBAL_QUERY);
	default:
	    throw new ADEIException("The ShowDatabases for \"" . $this->driver . "\" is not implemented");
    }
 } 
 
 function ShowTables() {
    switch ($this->driver) {
	case "mysql":
	    return $this->Query("SHOW TABLES");
	case "mssql":
		/* Bugs in ODBC/MySQL (with segm. in some cases,
		possibly http://bugs.php.net/bug.php?id=33533&edit=1
		*/
/*
	    $db = new DATABASE($this->server);
	    return $db->Query(" SELECT name AS gid FROM sysobjects WHERE type = 'U'");
*/

	    return $this->Query(" SELECT name AS gid FROM sysobjects WHERE (type = 'U' OR type = 'V')");
	default:
	    throw new ADEIException("The ShowTables for \"" . $this->driver . "\" is not implemented");
    }
 }

 function ShowColumns($table) {
    switch ($this->driver) {
	case "mysql":
	    return $this->Query("SHOW COLUMNS FROM `$table`");
	case "mssql":
		/* we could get here problems if several non-equal tables with 
		different prefixes (mda,dbo) are present */
	    $table = preg_replace("/^mda\./", "", $table);
	    return $this->Query("SELECT name FROM (SELECT DISTINCT TOP 65535 name=syscolumns.name, type=systypes.name, length=syscolumns.length, objname=sysobjects.name, colid=syscolumns.colid
		FROM sysobjects 
		JOIN syscolumns ON sysobjects.id = syscolumns.id 
		JOIN systypes ON syscolumns.xtype=systypes.xtype
		WHERE (sysobjects.xtype='U' OR sysobjects.xtype='V')  AND sysobjects.name='$table'
		ORDER BY sysobjects.name,syscolumns.colid) AS tmptable");
	default:
	    throw new ADEIException("SHOW COLUMNS not implemented for " . $this->driver);
    }
 }
 
 function CreateDatabase($dbname) {
    $this->Query("CREATE DATABASE `$dbname`", DATABASE::GLOBAL_QUERY);
 }
 
 
 function GetDatabaseList($filter = false) {
    $resp = $this->ShowDatabases();

    $dblist = array();

    foreach ($resp as $row) {
	$name = $row[0];
	if ((!$filter)||(preg_match($filter, $name))) {
	    $dblist[$name] = array(
		'name' => $name
	    );
	}
    }
    
    return $dblist;
 }
}


/* expecting rows in a form: chan1,chan2,...,chan#,time  */
class DATABASEData implements Iterator {
 var $reader;
 var $stmt;
 var $row;
 var $time;
 
 function __construct(READER &$reader, PDOStatement &$stmt) {
    $this->reader = &$reader;
    $this->stmt = &$stmt;
 }
 
 function rewind() {
    try {
	$this->stmt->execute();
	$this->next();
    } catch(PDOException $e) {
	throw new ADEIException(translate("SQL request is failed with error") . ": " . $e->getMessage(), $e->getCode());
    }

 }
 
    // current element
 function current() {
    return $this->row;
 }
 
    // current key (PHP rounding to integer :()
 function key() {
/*    echo $this->time;
    echo "  -  ";
    echo date('Y-m-d', $this->reader->ExportUnixTime($this->time));
    echo "\n";*/
    return $this->reader->ExportUnixTime($this->time);
 }
 
    // advvance to next (and returns it or false)
 function next() {
    try {
	$this->row = $this->stmt->fetch(PDO::FETCH_NUM);
	if ($this->row) {
	    $last = sizeof($this->row) - 1;
	    $this->time = $this->row[$last];
	    unset($this->row[$last]);
	}
    } catch(PDOException $e) {
	throw new ADEIException(translate("SQL error") . ": " . $e->getMessage());
    }
 }
 
    // checks if there is current element
 function valid() {
    return $this->row?true:false;
 }
}

?>