<?php

include($JPGRAPH_PATH . "/jpgraph.php");
include($JPGRAPH_PATH . "/jpgraph_line.php");
include($JPGRAPH_PATH . "/jpgraph_date.php");
include($JPGRAPH_PATH . "/jpgraph_canvas.php");
include($JPGRAPH_PATH . "/jpgraph_canvtools.php");


class DRAW {
 var $cache;
 var $req;
 var $interval, $mask;
 
 var $tmpfile;
 var $ready;
 
 var $width, $height;
 var $graph;
 var $graph_interval;

 var $use_cache_timewindow;
 var $set_time_axes;
 
 var $aggregator;		// Aggregating algorithm
 var $interpolate_gaps;		// Do not indicate missing data, do linear interpolation instead
 var $allowed_gap;		// The allowed gaps between data, report if greater (if no interpolation)

 var $show_marks;		// Set marks on the graph if data gaps are detected
 var $show_gaps;		// Present information about gaps in the data on graph

 var $precision;		// Actual graph precision in pixels
 
 const MARKS_NEVER = 0;
 const MARKS_DEFAULT = 1;
 const MARKS_GAPS = 2;
 const MARKS_ALWAYS = 3;
 
 const SHOW_NONE = 0;
 const SHOW_EMPTY = 1;
 const SHOW_POINTS = 2;
 const SHOW_GAPS = 3;


 function __construct(DATARequest &$props = NULL, CACHE &$cache = NULL) {
    global $GRAPH_DEFAULT_HEIGHT, $GRAPH_DEFAULT_WIDTH;
    global $TMP_PATH;

    global $GRAPH_INTERPOLATE_DATA_GAPS;
    global $GRAPH_ACCURACY_MARKS_IF_GAPS;
    
    global $GRAPH_INDICATE_MISSING_DATA;
    global $GRAPH_INDICATE_DATA_DENSITY;

    if ($props) $this->req = &$props;
    else $this->req = new DATARequest();

    if ($cache) $this->cache = &$cache;
    else $this->cache = $this->req->CreateCache();

    $opts = &$this->req->GetOptions();
    $this->use_cache_timewindow = $opts->Get('use_cache_timewindow');
    if ($this->use_cache_timewindow) {
	$this->interval = $this->req->CreateInterval();
	$this->interval->ApplyCache($this->cache);
    } else {
	$iinfo = $this->req->GetIntervalInfo();
        $this->interval = $this->cache->CreateInterval($iinfo);
    }
    
    $limit = $opts->GetDateLimit();
    $this->interval->Limit($limit[0], $limit[1]);

    $this->set_time_axes = !$opts->Get('optimize_time_axes');

    if (isset($this->req->props['aggregation']))
	$this->aggregator = $this->req->props['aggregation'];
    else
	$this->aggregator = CACHE::TYPE_AUTO;
    
    
    if (isset($this->req->props['show_marks']))
	$this->show_marks = $this->req->props['show_marks'];
    else {
	if ($GRAPH_ACCURACY_MARKS_IF_GAPS)
	    $this->show_marks = DRAW::MARKS_GAPS;
	else 
	    $this->show_marks = DRAW::MARKS_DEFAULT;
    }

    if (isset($this->req->props['show_gaps']))
	$this->show_gaps = $this->req->props['show_gaps'];
    else {
	if ($GRAPH_INDICATE_DATA_DENSITY) {
	    eval("\$this->show_gaps = DRAW::$GRAPH_INDICATE_DATA_DENSITY;");
	} else {
	    $this->show_gaps = $GRAPH_INDICATE_DATA_DENSITY;
	}
    }
    if (isset($this->req->props['interpolate']))
	$this->interpolate_gaps = $this->req->props['interpolate'];
    else
	$this->interpolate_gaps = $opts->Get('graph_interpolate', $GRAPH_INTERPOLATE_DATA_GAPS);
	
    $this->allowed_gap = $opts->Get('maximal_allowed_gap', false);
    
    
    $minfo = $this->req->GetMaskInfo();
    $this->mask = $this->cache->CreateMask($minfo);

    
    $this->tmpfile = $this->GetTmpFile();

/*  DS, ToDo: We should restore some variables before we can reuse files 
    if (is_file($TMP_PATH . "/" .  $this->tmpfile)) $this->ready = true;
    else */{
	if (isset($this->req->props['frame_height'])) $this->height = $this->req->props['frame_height'];
	else $this->height = $GRAPH_DEFAULT_HEIGHT;
	if (isset($this->req->props['frame_width'])) $this->width = $this->req->props['frame_width'];
	else $this->width = $GRAPH_DEFAULT_WIDTH;
	
	
	$this->ready = false;
    }
 }

 function GetTmpFile() {
    global $ADEI_SESSION;
    global $TMP_PATH;


    $props = $this->req->props;
    unset($props['window']);
    unset($props['format']);
    unset($props['mask_mode']);
    unset($props['resample']);
    unset($props['filename']);

    $dir = "clients/" . $ADEI_SESSION . "/draw." . $this->req->props['db_server'] . "__" . $this->req->props['db_name'] . "__" . $this->req->props['db_group'] . "/" . md5(serialize($props)) . "/";

    if (!is_dir($TMP_PATH . "/" .  $dir)) {
	if (!@mkdir($TMP_PATH . "/" . $dir, 0755, true)) 
	    throw new ADEIException(translate("DRAW class have not access to the temporary directory"));
    }


    return $dir . time() . "_" . $this->interval->window_start . "_" . $this->interval->window_size . ".png";
 }

 function GetAgregatingProperties($iv, $size) {
    global $GRAPH_MAX_POINTS_PER_GRAPH;
    global $GRAPH_MAX_APPROXIMATION_INTERVAL;
    global $GRAPH_AUTOAGGREGATION_MINMAX_THRESHOLD;
    
    $points = $this->width * $size;
    if ($points > $GRAPH_MAX_POINTS_PER_GRAPH) {
	$limit = (int)floor($this->width * $GRAPH_MAX_POINTS_PER_GRAPH / $points);

	$intivl = $this->width / $limit;
	if ($intivl > $GRAPH_MAX_APPROXIMATION_INTERVAL) {
	    $limit = (int)ceil($this->width / $GRAPH_MAX_APPROXIMATION_INTERVAL);
	}
    } else $limit = $this->width;

    $precision = (int)ceil(2 * $this->width / $limit);
    
    $res = $this->cache->resolution->Get($iv, $limit /* amount is here */);
    
    if ($this->aggregator) $aggregator = $this->aggregator;
    else {
	if ($precision < $GRAPH_AUTOAGGREGATION_MINMAX_THRESHOLD) $aggregator = CACHE::TYPE_MINMAX;
	elseif (!$this->cache->resolution->GetWindowSize($res)) $aggregator = CACHE::TYPE_MINMAX;
	else  $aggregator = CACHE::TYPE_MEAN;
    }
    
    if ($aggregator != CACHE::TYPE_MINMAX) {
	$precision = (int)ceil($this->width / $limit);
    }

    return array(
	"type" => $aggregator,
	"limit" => $limit,
	"amount" => $limit,
	"precision" => $precision,
	"resolution" => $res
    );
 }
 
 function Legend($x = false, $y = false, &$mask = NULL, INTERVAL &$ivl = NULL) {
    global $GRAPH_DELTA_SIZE;

    if ($ivl) $iv = $ivl;
    else $iv = $this->interval;

    if (!$mask) $mask = &$this->mask;

    $list = $this->cache->GetItemList($mask, 0);
    
    if ($x === false) {
	if (isset($this->req->props['x'])) $x = $this->req->props['x'];
	else throw new ADEIException(translate("The X coordinate is not specified"));

    }
    if ($y === false) {
	if (isset($this->req->props['y'])) $y = $this->req->props['y'];
	else throw new ADEIException(translate("The Y coordinate is not specified"));
    }
    
    if (isset($this->req->props['xmin'])) $xmin = $this->req->props['xmin'];
    else throw new ADEIException(translate("The XMin is not specified"));
    if (isset($this->req->props['xmax'])) $xmax = $this->req->props['xmax'];
    else throw new ADEIException(translate("The XMax is not specified"));
    if (isset($this->req->props['ymin'])) $ymin = $this->req->props['ymin'];
    else throw new ADEIException(translate("The YMin is not specified"));
    if (isset($this->req->props['ymax'])) $ymax = $this->req->props['ymax'];
    else throw new ADEIException(translate("The yMax is not specified"));


    $size = sizeof($this->mask->ids);
    $a = $this->GetAgregatingProperties($iv, $size);
    
    if ($a["limit"] > 0) {
	/*
	    "4" is due to the fact what maximal approximation error is two
	    intervals. See description in cache.php.
	    FIXME: limit could be decreased due to the border points
	*/

	if ($a['type'] == CACHE::TYPE_MINMAX)
	    $delta_size = 4 * $this->width / $a["limit"]; 
	else
	    $delta_size = 2 * $this->width / $a["limit"]; 

	if ($delta_size < $GRAPH_DELTA_SIZE)
	    $delta_size = $GRAPH_DELTA_SIZE;
    } else {
    	$delta_size = $GRAPH_DELTA_SIZE;
    }
    
//    echo "$delta_size ($GRAPH_DELTA_SIZE)\n";
//    exit;
    
//    echo date('c',$x) . "(" . date('c',$xmin) . " - " . date('c', $xmax) . ")\n";

    $iv->SetupInterval($xmin, $xmax);
    $iv->SetupWindow(INTERVAL::WINDOW_NEIGHBOURHOOD, $x,  (dsMathPreciseSubstract($xmax, $xmin) * $delta_size) / $this->width);
//    echo date('c',(int)$iv->window_start) . " - " . date('c',(int)$iv->GetWindowEnd())  . "\n";

/*    
    $iv->window_size = (dsMathPreciseSubstract($xmax, $xmin) * $delta_size) / $this->width;
    $iv->window_start = dsMathPreciseSubstract($x, $iv->window_size / 2);
    if ($iv->window_start < $xmin) {
	$iv->window_size -= dsMathPreciseSubstract($xmin, $iv->window_start);
	$iv->window_start = $xmin;
    }
*/
    
    $ywin = (($ymax - $ymin)  * $GRAPH_DELTA_SIZE) / $this->height;
    $min = $y - $ywin / 2;
    $max = $y + $ywin / 2;
    
    
    $flags = CACHE::TRUNCATE_INTERVALS;
    $iv->EnableFlexibility();
//    print_r($a);
//    exit;
    $points = $this->cache->GetIntervals($mask, $iv, 1, 0, $a["resolution"], $flags);

/*
    $f=fopen("/tmp/xxx6", "w");
    fprintf($f, "$x $y ($delta_size), [" . $this->width . "x" . $this->height . "]\n");
    fprintf($f, "$xmin - $xmax, $ymin:$ymax)\n");
    fprintf($f, "$resolution\n");
    fprintf($f, print_r($a,true));
    fprintf($f, print_r($mask, true));
    fprintf($f, print_r($iv, true));
    fprintf($f, print_r($list, true));
    fprintf($f, "ID's: " . $size . "\n");
*/

    $result = array(); $notfound = true;
    foreach($points as $t => $v) {
//	fwrite($f, $t . ": ");
//	fwrite($f, print_r($v, true));
	
//	echo date('c', $t) . "\n";
//	print_r($v); 
//	echo "!\n";

	for ($i=0;$i<$size;$i++) {
	    if ((($v["max" .$i]<$min)||($v["min" .$i]>$max))==false) {
		array_push($result, array(
		    "id" => $i,
		    "name" => xml_escape($list[$i]['name'])
		));
	    }
	}
	$notfound = false;
	break;
    }
    
    if ($notfound) {
	$neighbors = $this->cache->GetNeighbors($mask, $x);
	if ((sizeof($neighbors['left']) > 0)&&(sizeof($neighbors['right']) > 0)) {
	    $keys1 = array_keys($neighbors['left']); $x1=$keys1[0];
	    $keys2 = array_keys($neighbors['right']);$x2=$keys2[0];
//	    echo "X: $x ($xmin, $xmax)\n";
//	    echo "Y: $y ($min, $max)\n";

//	    echo "X: $x1, $x, $x2\n";

	    $check1 = true;
	    if ((!$this->interpolate_gaps)&&($this->allowed_gap)) {
		    /* We don't check expected gap, since otherwise this points
		    would be found before neighbors lookup */
		if ((dsMathPreciseSubstract($x, $x1)>$this->allowed_gap)||(dsMathPreciseSubstract($x2, $x)>$this->allowed_gap)) $check1 = false;
	    }

	    if (($check1)&&(dsMathPreciseCompare($x1, $xmin)>=0)&&(dsMathPreciseCompare($x2, $xmax)<=0)) {
	        $coef = dsMathPreciseSubstract($x, $x1) / dsMathPreciseSubstract($x2, $x1);
//		echo $coef . "(" . $size . ")\n";
		for ($i=0;$i<$size;$i++) {
		    $y1 = $neighbors['left'][$x1][$i];
		    $y2 = $neighbors['right'][$x2][$i];
		    $y = $y1 + $coef*($y2 - $y1); //DS.Precision
		
//		    print "Y: $y1 - $y - $y2\n";
		    if (($y>$min)&&($y<$max)) {
			array_push($result, array(
			    "id" => $i,
			    "name" => xml_escape($list[$i]['name'])
			));
		    }
		}
	    }
	}
    }
    
/*    
//    fprintf($f, print_r($ivl, true));
    fprintf($f, print_r($result, true));
    fclose($f);
*/
    
    return array("legend" => $result);
 }
 
 function Create($mask = NULL, INTERVAL $iv = NULL) {
    global $GRAPH_MARGINS;
    global $GRAPH_MAX_CACHE_GAP;
    global $GRAPH_COLORS;
    global $GRAPH_LINE_WEIGHT;
    global $GRAPH_ACCURACY_MARKS_OUTSET;
    global $GRAPH_ACCURACY_MARKS_MULTIOUTSET;
    global $GRAPH_ACCURACY_MARKS_COLOR;
    global $GRAPH_ACCURACY_MARKS_TYPE;
    global $GRAPH_ACCURACY_MARKS_SIZE;
    global $GRAPH_SUBSECOND_THRESHOLD;
    
    global $GRAPH_DENSITY_PLOT_VALID_SIZE;
    global $GRAPH_DENSITY_PLOT_VALID_COLOR;
    global $GRAPH_DENSITY_PLOT_INVALID_SIZE;
    global $GRAPH_DENSITY_PLOT_INVALID_COLOR;
    global $GRAPH_DENSITY_POINTS_TYPE;
    global $GRAPH_DENSITY_POINTS_SIZE;
    global $GRAPH_DENSITY_POINTS_COLOR;
    global $GRAPH_DENSITY_POINTS_OUTLINE;
    
    

    if ($this->ready) return;
    if (!$iv) $iv = $this->interval;
    if (!$mask) $mask = $this->mask;

    $size = sizeof($mask->ids);
    $a = $this->GetAgregatingProperties($iv, $size);
    $precision = $a['precision'];
/*    
    if ($a['limit']) $precision = (int)ceil(2 * $this->width / $a["limit"]);
    else $precision = 0;
*/
//    print_r($iv);
    $flags = ($this->interpolate_gaps?0:CACHE::REPORT_EMPTY)|(($this->show_gaps==DRAW::SHOW_GAPS)?CACHE::MISSING_INFO:0);
    $points = $this->cache->GetPoints($mask, $iv, $a['type'], $a['limit'], $a['amount'], $a['resolution'], $flags);
    if ($this->allowed_gap !== false) $points->SetOption("allowed_gap", $this->allowed_gap);

/*    
    if ($size > 1) {
	if ($size > 10) $points = 100;
	else if ($size > 5) $points = 20;
	else if ($size > 3) {
	    $points = $this->width / $size;
	}
    
//	$points = $this->cache->GetPoints($mask, $iv, CACHE::TYPE_MINMAX);
	$points = $this->cache->GetPoints($mask, $iv, CACHE::TYPE_MINMAX, $points);
    } else {
	$points = $this->cache->GetPoints($mask, $iv, CACHE::TYPE_MINMAX, $this->width);
    }
*/
    
    $amount = 0;
    $time = array();
    $values = array();
    
    $missing = true; 		// last point was missing
    $amount_on_missing = false;
    $time_on_addon = false;
    
    for ($i=0;$i<$size;$i++)
	$values[$i] = array();


    if (($iv->y_min)||($iv->y_max)) {
	$min = $iv->y_min;
	$max = $iv->y_max;
	
	for ($i=0;$i<$size;$i++) {
	    $at_max[$i] = false;
	    $at_min[$i] = false;
	}
    } else {
	$min = 0;
	$max = 0;

        if ($this->show_gaps) {
	    $realmin = 1E+200;
	    $realmax = -1E+200;
	}
    }
    
    if ($this->show_gaps) {
        $mvalue = array();
    }
    
    

    foreach($points as $t => $v) {
	if ($v) {
	    if ($this->show_gaps) {
		$gapinfo = $points->missing_data();
		if ($gapinfo) array_push($mvalue, 1);
		else array_push($mvalue, 0);
	    }

	    $amount++;

	    if ($missing) {
/*		if ($time_on_addon) {
		    if ($this->show_gaps) {
			array_push($mvalue, 2);
		    }

		    array_push($time, $time_on_addon);
		    for ($i=0;$i<$size;$i++) {
		        array_push($values[$i], false);
		    }		  
			
		    $time_on_addon = false;  
		}*/
		    
		$amount_on_missing = $amount;
		$missing = false;
	    }


	    array_push($time, $t);
	    if (($min)||($max)) {
		foreach ($v as $key => $value) {
		    if ($at_max[$key]) {
			if ($value > $max) {
			    $value = "";
			} else {
			    $at_max[$key] = false;
			    if ($value < $min) {
				$value = $min;
				$at_min[$key] = true;
			    }
			}
		    } elseif ($at_min[$key]) {
			if ($value < $min) {
			    $value = "";
			} else {
			    $at_min[$key] = false;
			    if ($value > $max) {
				$value = $max;
				$at_max[$key] = true;
			    }
			}
		    } else {
			if ($value > $max) {
			    $value = $max;
			    $at_max[$key] = true;
			} elseif ($value < $min) {
			    $value = $min;
			    $at_min[$key] = true;
			}
		    }
		    array_push($values[$key], $value);
		}
	    } else {
		if ($this->show_gaps) {
		    foreach ($v as $key => $value) {
			if ($value < $realmin) $realmin = $value;
			if ($value > $realmax) $realmax = $value;
			array_push($values[$key], $value);
		    }
		} else {
		    foreach ($v as $key => $value) {
			array_push($values[$key], $value);
		    }
		}
	    }
	} else {
	    if (!$missing) {
		if ($amount === $amount_on_missing) {
		    if ($this->show_gaps) {
			    /* Yes, this point is set only to prolong last single (standalone) point to interval,
			    so it would be visualized by jpgraph. But in reality it is not a point */
			array_push($mvalue, 2);
		    }

		    $pos = sizeof($values[0]) - 1;
		    array_push($time, $time[$pos]);
		    for ($i=0;$i<$size;$i++) {
			array_push($values[$i], $values[$i][$pos]);
		    }
			
//		    $time_on_addon = $t;
		    $amount_on_missing = false;
		}
		
		if ($this->show_gaps) {
		    array_push($mvalue, 2);
		}

//		$time_on_addon = false;

		array_push($time, $t);
		for ($i=0;$i<$size;$i++) {
		    array_push($values[$i], false);
		}


		$missing = true;
	    }
	}
    }
    
    if ((!$missing)&&($amount === $amount_on_missing)) {
    	if ($this->show_gaps) {
	    array_push($mvalue, 2);
	}

	$pos = sizeof($values[0]) - 1;
	array_push($time, $time[$pos]);//$t);
	for ($i=0;$i<$size;$i++) {
	    array_push($values[$i], $values[$i][$pos]);
	}
    }
    
//    echo "Points detected: $amount(" . sizeof($time) . ")\n";
    
    if (!$amount) {
	    // Restoring request interval
	if ($iv == $this->interval) {
	    $iv = $this->req->CreateInterval();
	} 
	
	$grinfo = $this->cache->reader->GetGroupInfo($this->cache->group);
	$grname = $grinfo['name'];
	$dbname = $this->cache->reader->dbname;
	
	if ($iv->IsEmpty()) {
	    throw new ADEIException(translate("The group '%s' from database '%s' contains no data at all", $grname, $dbname, $from, $to), ADEIException::NO_DATA);
	} else {
	    $from = date('c',(int)$iv->GetWindowStart()); 
	    $to = date('c',(int)$iv->GetWindowEnd());

	    throw new ADEIException(translate("The group '%s' from database '%s' have no data available between %s and %s ", $grname, $dbname, $from, $to), ADEIException::NO_DATA);
	}
    }


    if ($amount != sizeof($time)) {
	$have_gaps = true;
    } else {
	$have_gaps = $points->missing_points();
    }


    if ($this->show_gaps == DRAW::SHOW_GAPS) $show_gaps = DRAW::SHOW_GAPS;
    else if (($this->show_gaps == DRAW::SHOW_POINTS)||(($have_gaps)&&($this->show_gaps == DRAW::SHOW_EMPTY))) $show_gaps = DRAW::SHOW_POINTS;
    else $show_gaps = 0;

    if ($show_gaps) {
	if ((!$min)&&(!$max)) {
	    if ($realmin == $realmax) {
		if ($realmin > 0) {
	    	    $px = $realmax / $this->height;
		    $min = 0;
		    $max = 2*$realmax;
		} elseif ($realmin < 0) {
		    $px = (-$realmax) / $this->height;
		    $min = 2*$realmin;
		    $max  = 0;
		} else {
		    $px = 1 / $this->height;
		    $max = 1;
		    $min = -1;
		}
	    } else {
		$px = ($realmax - $realmin) / $this->height;
		$min = $realmin - 4 * $px;
		$max = $realmax + 4 * $px;
	    }
	} else {
	    $px = ($max - $min) / $this->height;
	}

	$realmax = $max + 2*$px;
	
//	echo "$min ($realmin)  , $max ($realmax) \n";
    }


//    print_r($time);

    $from = $time[0];
    $to = $time[sizeof($time)-1];
    $length = dsMathPreciseSubstract($to, $from);
    
	// Calculating real accuracy, but only if single plot, otherwise not important    
//    if ($size == 1) {
	if ($aggregator == CACHE::TYPE_MINMAX) {
	    $precision = (int)ceil(2 * $this->width / $amount);
	} else {
	    $precision = (int)ceil($this->width / $amount);
	}
//    }

    if ($GRAPH_MAX_CACHE_GAP) {
	if ((($from - $iv->window_start) < $GRAPH_MAX_CACHE_GAP)||
	    (($to - $iv->window_start - $iv->window_size) < $GRAPH_MAX_CACHE_GAP)) {
	    $range1 = date("c", ceil($iv->window_start)) . " and " . date("c", floor($iv->window_start + $iv->window_size));
	    $range2 = date("c", ceil($from)) . " and " . date("c", floor($to));
	    throw new ADEIException(translate("The data is missing in CACHE: Needed: $range1, Available: $range2"));
	}
    }



    $this->graph = new Graph($this->width, $this->height, "auto");
    $this->graph->img->SetMargin($GRAPH_MARGINS['left'],$GRAPH_MARGINS['top'],$GRAPH_MARGINS['right'],$GRAPH_MARGINS['bottom']);
    $this->graph->SetTickDensity(TICKD_NORMAL, TICKD_SPARSE);

//    $this->graph->SetTickDensity(TICKD_SPARSE, TICKD_VERYSPARSE);
//    $this->graph->title->Set("Example on Date scale");
//    $this->graph->xaxis->scale->SetTimeAlign(MINADJ_15);
//    $this->graph->xscale->SetGrace(10)

    if ($length > $GRAPH_SUBSECOND_THRESHOLD) {
	if (($this->set_time_axes)||($from == $to)) {
	    $from = $iv->GetWindowStart();
	    $to = $iv->GetWindowEnd();
	    
	    if (($min)||($max)) $this->graph->SetScale("datlin", $min, $max, $from, $to);
	    else $this->graph->SetScale("datlin", true, true, $from, $to);

	    $length = $iv->window_size;
	} else {
	    if (($min)||($max)) $this->graph->SetScale("datlin", $min, $max);
	    else $this->graph->SetScale("datlin");
	}

	$this->scale_start = 0;
	$this->scale_coef = 1;
    } else {
	if (($this->set_time_axes)||($from == $to)) {
	    $from = $iv->window_start;
	    $to = dsMathPreciseAdd($iv->window_start, $iv->window_size);

	    $length = $iv->window_size;

	    if ($length > $GRAPH_SUBSECOND_THRESHOLD) {
		if (($min)||($max)) $this->graph->SetScale("datlin", $min, $max, $from, $to);
		else $this->graph->SetScale("datlin", true, true, $from, $to);
	    
		$this->scale_start = 0;
		$this->scale_coef = 1;
	    }
	}
    }


    $afrom = getdate($from);
    $ato = getdate($to);

    if ($length > 315360000) { // 10 years 
        $this->graph->xaxis->scale->SetDateFormat('Y');
	$this->graph->xaxis->SetTextLabelInterval(1);
	$date_title = "";
    } elseif ($length > 31104000) { // 1 year
	
	if ($afrom['year'] == $ato['year']) {
	    $this->graph->xaxis->scale->SetDateFormat('M');
	    $this->graph->xaxis->SetTextLabelInterval(2);
	    
	    $date_title = $afrom['year'];
	} else {
	    $this->graph->xaxis->scale->SetDateFormat('M, Y');
	    $this->graph->xaxis->SetTextLabelInterval(2);
	    
	    $date_title = $afrom['year'] . " - " . $ato['year'];
	}
    } elseif ($length > 1036800) { // 12 days
        $this->graph->xaxis->scale->SetDateFormat('M d');
	$this->graph->xaxis->SetTextLabelInterval(3);
	
	if ($afrom['year'] == $ato['year']) {
	    if ($afrom['mon'] == $ato['mon']) {
		$date_title = $afrom['month'] . ", " . $afrom['year'];
	    } else {
		$date_title = $afrom['year'];
	    }
	} else {
	    $date_title = $afrom['year'] . " - " . $ato['year'];
	}
    } elseif ($length > 86400) { // 1 day
        $this->graph->xaxis->scale->SetDateFormat('M d, H:i');
	$this->graph->xaxis->SetTextLabelInterval(4);

	if ($afrom['year'] == $ato['year']) {
	    if ($afrom['mon'] == $ato['mon']) {
		$date_title = $afrom['month'] . ", " . $afrom['year'];
	    } else {
		$date_title = $afrom['year'];
	    }
	} else {
	    $date_title = $afrom['year'] . " - " . $ato['year'];
	}
    } elseif ($length > 14400) { // 4 hours
	$this->graph->xaxis->SetTextLabelInterval(2);
        $this->graph->xaxis->scale->SetDateFormat('H:i');

	if ($afrom['year'] == $ato['year']) {
	    if ($afrom['mon'] == $ato['mon']) {
		if ($afrom['mday'] == $ato['mday']) {
		    $date_title = $afrom['month'] . " " . $afrom['mday'] . ", " . $afrom['year'];
		} else {
		    $date_title = $afrom['month'] . " " . $afrom['mday'] . " - " . $ato['mday'] . ", " . $afrom['year'];
		}
	    } else {
		$date_title = date("M", $from) . " " . $afrom['mday'] . " - " . date("M", $to) . " " . $ato['mday'] . ", " . $afrom['year'];
	    }
	} else {
	    $date_title = date("M j, Y", $from) . " - " . date("M j, Y", $to);
	}
    } else if ($length > $GRAPH_SUBSECOND_THRESHOLD) {
	if ($length > 10) $this->graph->xaxis->SetTextLabelInterval(4);
	else $this->graph->xaxis->SetTextLabelInterval(4);
        $this->graph->xaxis->scale->SetDateFormat('H:i:s');

	if ($afrom['year'] == $ato['year']) {
	    if ($afrom['mon'] == $ato['mon']) {
		if ($afrom['mday'] == $ato['mday']) {
		    $date_title = $afrom['month'] . " " . $afrom['mday'] . ", " . $afrom['year'];
		} else {
		    $date_title = $afrom['month'] . " " . $afrom['mday'] . " - " . $ato['mday'] . ", " . $afrom['year'];
		}
	    } else {
		$date_title = date("M", $from) . " " . $afrom['mday'] . " - " . date("M", $to) . " " . $ato['mday'] . ", " . $afrom['year'];
	    }
	} else {
	    $date_title = date("M j, Y", $from) . " - " . date("M j, Y", $to);
	}
    } else {
	$ifrom = floor($from);
	if (is_float($from)) $rfrom = substr(printf("%.9F", $from - $ifrom),2);
	else {
	    $pos = strpos($from, ".");
	    if ($pos === false) $rfrom = 0;
	    else $rfrom = substr($from, $pos + 1);
	}

	$date_title = date("M j, Y H:i:s", $ifrom);
	if ($rfrom) {
	    $date_title .= "." . $rfrom;
	    $rfrom = "0.$rfrom";
	}
	
	if ($length > $this->width/1000)  {
	    $coef = 1000;
	    $suffix = "ms";
	}
	elseif ($length > $this->width/1000000)  {
	    $coef = 1000000;
	    $suffix = "us";
	} else {
	    $coef = 1000000000;
	    $suffix = "ns";
	}

	$reallength = (int)floor($length*$coef);
	$rem = ($reallength + 10) % 1000;
	if ($rem<20) $ilength = $reallength - ($rem - 10);
	else $ilength = $reallength;
	
//	echo $reallength . " - " . $length . "\n";
	
	$date_title .= " + " . $ilength . $suffix;

//	echo $coef . " - " . $ilength .  "(" . $length . ")\n";
	$first_num = substr($ilength, 0, 1);
	$second_num = substr($ilength, 1, 1);
	if ($second_num>4) {
	    if ((++$first_num)==10)
		$istep = "1" . str_repeat("0", strlen($ilength) - 1);
	    else 
		$istep = $first_num . str_repeat("0", strlen($ilength) - 2);
	} else $istep = $first_num . str_repeat("0", strlen($ilength) - 2);

	if (($min)||($max)) $this->graph->SetScale("linlin", $min, $max, 0, $reallength); // + 1);
	else $this->graph->SetScale("linlin", true, true, 0, $reallength);// + 1);
	$this->graph->xscale->ticks->Set($istep, $istep/4);
	
	$ticks = floor(($ilength+1) / $istep);

	if ($this->width < 350) {
	    if ($this->width < 70) $lpt = $ticks;
	    else $lpt = ceil ($ticks / ($this->width / 70));
	} else $lpt = round($ticks / 5);
	
	$this->graph->xaxis->SetTextLabelInterval($lpt);
	
	foreach ($time as &$t) {
	    strstr($t, ".");

	    $it = (int)floor($t);
	    $pos = strpos($t, ".");
	    if ($pos === false) $rt = 0;
	    else $rt = "0." . substr($t, $pos + 1);
	    
//	    echo $coef . ": " . $t;
	    $t = ($it - $ifrom)*$coef + floor(($rt - $rfrom)*$coef);
//	    echo " - " . $t . "\n";
	}
	
	$this->scale_start = $from;
	$this->scale_coef = $coef;

//	$this->graph->xaxis->SetTextLabelInterval(1);
    }

    $title = $this->req->props['db_name'];
/*  Full location information. Too long
    $title = $this->req->props['db_server'] . ":" . $this->req->props['db_name'] . ":" . $this->req->props['db_group'];
*/	
/*    Precision information. Confusing 
*/
    $operation_info = $points->GetOperationInfo();
    
    if (isset($operation_info["resolution"])) $title .= ", res: " . $operation_info["resolution"];
    if (isset($operation_info["aggregation"])) {
	switch ($operation_info["aggregation"]) {
	    case CACHE::TYPE_MINMAX:
		$title .= ", MMAX";
	    break;
	    case CACHE::TYPE_MEAN:
		$title .= ", MEAN";
	    break;
	}
    }
    if ($precision) $title .= ", acc. " . $precision . "px";
    if ($date_title) $title = "$date_title ($title)";
    
    $this->graph->xaxis->SetFont(FF_ARIAL, FS_NORMAL, 8);
//    $this->graph->xaxis->SetLabelAngle(0);

	// We can use SetLabelFormatCallback for higher control
    $this->graph->yaxis->SetLabelFormat('%0.5g');


    $this->graph->title->Set($title);
    $this->graph->xaxis->SetPos("min");


    switch ($this->show_marks) {
	case DRAW::MARKS_ALWAYS:
	    $marks = true;
	break;
	case DRAW::MARKS_GAPS:
	    if ($have_gaps) {
		$marks = true;
		break;
	    }
	case DRAW::MARKS_DEFAULT:
	    if ($size > 1) {
		if (($GRAPH_ACCURACY_MARKS_MULTIOUTSET)&&($precision > $GRAPH_ACCURACY_MARKS_MULTIOUTSET)) {
		    $marks = true;
		    break;
		}
	    } else {
		if (($GRAPH_ACCURACY_MARKS_OUTSET)&&($precision > $GRAPH_ACCURACY_MARKS_OUTSET)) {
		    $marks = true;
		    break;
		}
	    }
	case DRAW::MARKS_NEVER:
	    $marks = false;
    }
	

    if ($marks) {
	if ($GRAPH_ACCURACY_MARKS_COLOR) $marks = $GRAPH_ACCURACY_MARKS_COLOR;
	else $marks = "black";

	if ($GRAPH_ACCURACY_MARKS_TYPE) {
	    eval("\$mtype=$GRAPH_ACCURACY_MARKS_TYPE;");
	} else {
	    $mtype = MARK_FILLEDCIRCLE;
	}
    }

    $no_data = true;
    for ($i = 0; $i < $size; $i++) {
/*
	$f = fopen("/tmp/xxx.7", "a+");
	fwrite($f, print_r($iv, true) . "\n");
	fwrite($f, $i . " of " . $size . "\n");
	fwrite($f, date("r", ceil($iv->window_start)) . " - " . date("r", floor($iv->window_start + $iv->window_size)) . "\n");
	fwrite($f, print_r($time, true));
	fwrite($f, print_r($values[$i], true));
	fclose($f);
*/
	if (sizeof($time) > 0) {

	    $empty_data = true;
	    foreach ($values[$i] as $val) {
		if ($val !== NULL) {
		    $empty_data = false;
		    break;
		}
	    }

	    if ($empty_data) continue;
	
	
	    $plot = new LinePlot($values[$i], $time);
	    if (sizeof($GRAPH_COLORS)) {
		$color = $GRAPH_COLORS[$i%sizeof($GRAPH_COLORS)];
		$plot->SetColor($color);
		if ($GRAPH_LINE_WEIGHT) {
		    $plot->SetWeight($GRAPH_LINE_WEIGHT);
		}
	    }
	    
	    
	    if ($marks) {
		$plot->mark->SetType($mtype);
		if ($GRAPH_ACCURACY_MARKS_SIZE) $plot->mark->SetSize($GRAPH_ACCURACY_MARKS_SIZE);
		$plot->mark->SetFillColor(($color&&($size>1))?$color:$marks);
	    }
	    
	    $this->graph->Add($plot);
	    $no_data = false;
	}
    }
    
    if ($no_data) {
	$range = date("c", ceil($iv->window_start)) . " and " . date("c", floor($iv->window_start + $iv->window_size));
	throw new ADEIException(translate("No data between: %s", $range), ADEIException::NO_DATA);
    }

    if ($show_gaps == DRAW::SHOW_GAPS) {
	$gtime = array();
	$gvalue = array();
	
	$flag = 0;
	foreach ($mvalue as $idx => &$val) {
	    if ($val) {
		if ($flag) $flag++;
		else {
		    array_push($gtime, $time[$idx]);
		    array_push($gvalue, $realmax);
		    $flag = 1;
		}
	    } else {
		if ($flag) {
		    array_push($gtime, $time[$idx]);
		    array_push($gvalue, $realmax);
		    
		    array_push($gtime, $time[$idx]);
		    array_push($gvalue, false);
		    
		    $flag = 0;
		}
	    }
	}
	
	if (sizeof($gtime) > 0) {
	    $plot = new LinePlot($gvalue, $gtime);
	    $plot->SetColor($GRAPH_DENSITY_PLOT_INVALID_COLOR);
	    $plot->SetWeight($GRAPH_DENSITY_PLOT_INVALID_SIZE);
	    $this->graph->Add($plot);
	}
    } else if ($show_gaps == DRAW::SHOW_POINTS) {
	foreach ($mvalue as &$val) {
	    if ($val>1) $val = false;
	    else $val = $realmax;
	}

	$plot = new LinePlot($mvalue, $time);
	$plot->SetColor($GRAPH_DENSITY_PLOT_VALID_COLOR);
	$plot->SetWeight($GRAPH_DENSITY_PLOT_VALID_SIZE);

	if ($GRAPH_DENSITY_POINTS_TYPE) {
	    eval("\$mtype=$GRAPH_DENSITY_POINTS_TYPE;");
	} 


	$plot->mark->SetType($mtype);
	$plot->mark->SetColor($GRAPH_DENSITY_POINTS_OUTLINE);
	$plot->mark->SetFillColor($GRAPH_DENSITY_POINTS_COLOR);
	$plot->mark->SetSize($GRAPH_DENSITY_POINTS_SIZE);
	$this->graph->Add($plot);

    }

    $this->graph_interval = $iv;
    $this->precision = $precision;
    
//    print_r($mvalue);
//    echo $realmin . "\n";

	
    
/*    $graph->img->SetMargin(40,110,20,40);
    //$graph ->xaxis->scale-> SetDateFormat( 'Y-m-d');
    $graph->xaxis->SetFont(FF_ARIAL,FS_NORMAL,6); 
    $graph->xaxis->SetLabelAngle(45);
    $graph->SetShadow();

    $graph->ygrid->Show(true,true);
$Size = sizeof($Data->TimeSeries);
for($i=3; $i<6;$i++){
    $lineplot[$i]=new LinePlot($Data->TimeSeries[$i]["Value"], $Data->TimeSeries[$i]["Time"]);
    $lineplot[$i]->SetColor($Data->TimeSeries[$i]["Color"]);
    $lineplot[$i]->SetLegend($Data->TimeSeries[$i]["Name"]);
    $graph->Add($lineplot[$i]);
}
$graph->Stroke($TempDir."Lin-".$FileName.".png");
    
*/
 
 
 
//        $graph = new Graph($this->width, $this->heigh /*, "auto"*/);
//	$graph->SetScale('intlin'/* ymin, ymax, xmin, xmax */);
	
	// foreach masks
//	$lineplot = new LinePlot($dataarray);
 }
 
 
 function Save($file = false) {
    global $TMP_PATH;

    if ($this->ready) {
	if ($file) {
	    copy($TMP_PATH . "/" .  $this->tmpfile, $file);
	    return true;
	}

	return $this->tmpfile;
    }

    if ($file) {
        $this->graph->Stroke($file);
	return true;
    }

    $this->graph->Stroke($TMP_PATH . "/" . $this->tmpfile);
    return $this->tmpfile;
 }


 function CreateMessage($header, $msg) {
    $this->graph = new CanvasGraph($this->width, $this->height, 'auto');
    $this->graph->SetMargin(5,11,6,11);
    $this->graph->SetShadow();
    $this->graph->SetMarginColor( "teal");
    $this->graph->InitFrame(); 
    
    $text_width = $this->width - 50;
    if ($text_width < 100) return;

    $text = new Text($msg, 25, 25);
    $text->SetFont(FF_ARIAL, FS_NORMAL, 24);

//    $text->Align('left', 'top');
//    $text->ParagraphAlign('left'); 
//    $text->SetBox( "white", "black","gray"); 
    
    $width = $text->GetWidth($this->graph->img);
    
    if ($width > $text_width) {
	$char_width = ceil($width / strlen($msg));
	$cpl = $text_width / $char_width;
/*
	Does not taken into the account by GetWidth function
	$text->SetWordWrap($cpl);
*/

	$wmsg = wordwrap($msg, $cpl, "\n", true);
	$text->Set($wmsg);
	
	$width = $text->GetWidth($this->graph->img);
        while (($width > $text_width)&&($cpl>10)) {
	    $cpl-=$cpl/10;
	    $wmsg = wordwrap($msg, $cpl, "\n", true);
	    $text->Set($wmsg);
	    $width = $text->GetWidth($this->graph->img);
	}
    }
    
    $text->Stroke( $this->graph->img); 
 }
 


 function GetScaleInfo() {
    global $GRAPH_MARGINS;

/*    if (!$this->scale_coef) {
	$f = fopen("/tmp/xxx.10", "w");
	fwrite($f, print_r($this, true));
	fclose($f);
    }*/
    return array(
//	"a" => date("c", $this->graph->xaxis->scale->scale[1]),
	"xmin" => dsMathPreciseAdd($this->graph->xaxis->scale->scale[0] / $this->scale_coef, $this->scale_start),
	"xmax" => dsMathPreciseAdd($this->graph->xaxis->scale->scale[1] / $this->scale_coef, $this->scale_start),
	"ymin" => $this->graph->yaxis->scale->scale[0],
	"ymax" => $this->graph->yaxis->scale->scale[1],
	"imin" => $this->graph_interval->GetIntervalStart(),
	"imax" => $this->graph_interval->GetIntervalEnd(),
	"yzoom" => (($this->graph_interval->y_min)||($this->graph_interval->y_min)),
	"precision" => $this->precision,
	"margins" => array($GRAPH_MARGINS['left'],$GRAPH_MARGINS['top'],$GRAPH_MARGINS['right'],$GRAPH_MARGINS['bottom'])
    );
 }
 
 static function Display($file = false) {
    global $TMP_PATH;
    
    if ($file) {
	if (preg_match("/^[A-Za-z0-9\/_]\.png$/",$str)) return false;
	return @readfile($TMP_PATH . "/" . $file);
    }

    return false;    
 }
}

?>