<?php

class INTERVAL {
 var $interval_start, $interval_end, $interval_items;
 var $window_start, $window_size, $window_items;
 var $y_min, $y_max;
 
 
 
 const NAME_FORMAT_DEFAULT = 0;
 const NAME_FORMAT_HUMAN = 0;
 const NAME_FORMAT_ISO8601 = 1;
 
 /*
    We can configure for types of windows:
    <---> 	no limits (all_mode, end_mode)
    ---->	specified window_size from the end (end_mode)
    |--->	specified window_start without size limit (all_mode)
    |--|	specified start and size limits ()
    
    WINDOW:
	specified size 	=> !end_mode
	specified start => !all_mode

    INTERVAL:
	specified begin => !end_mode
	specified_end => !end_mode, !all_mode
	
    end_mode:	INTERVAL: no start, no end	WINDOW: no start
    all_mode:	INTERVAL: no_end 		WINDOW: no size
    
    mode = 2, Unlocked by INTERVAL and WINDOW
    mode = 1, Unlocked by INTERVAL, but locked by WINDOW
    mode = 0, Locked by INTERVAL
    
    We expecting after Apply, the intervals are only allowed to grow.
 */
 
 var $end_mode;
 var $all_mode;
 
 var $sequence;
 
// var $flags;
 var $flexibility;

 const NEED_AMOUNT = 1;
 const FLAG_FLEXIBLE = 2;
 const FLAG_REDUCIBLE = 6;

 const WINDOW_RIGHT = 1;
 const WINDOW_LEFT = 2;
 const WINDOW_NEIGHBOURHOOD = 3;	// Diameter as a size
 const WINDOW_MAXIMUM = 4;

 const SEQUENCE_AUTO = 1;
 const SEQUENCE_FORWARD = 1;
 const SEQUENCE_BACKWARD = 2;
 const SEQUENCE_UNSORTED = 3;
 const SEQUENCE_RANDOM = 4;
 
 const MODE_LOCKED = 0;			// Exact value is defined (in experiment)
 const MODE_WINDOWLOCKED = 1;		// Locked by Window
 const MODE_UNLOCKED = 2;
 

 function __construct(array &$props = NULL, READER &$reader = NULL, LOGGROUP &$grp = NULL, $flags = 0) {
    if (($props)&&(isset($props['experiment']))&&(preg_match("/(\d+)-(\d+)/", $props['experiment'], $m))) {
        $this->interval_start = $m[1];
	$this->interval_end = $m[2];
	
	if ($m[2]) {
	    if ($m[1]) {
		$this->all_mode = INTERVAL::MODE_LOCKED;
		$this->end_mode = INTERVAL::MODE_LOCKED;
	    } else {
		throw new ADEIException(translate("Invalid INTERVAL '%s': have end but no start", $props['experiment']));
	    }
	} else {
	    if ($m[1]) $this->end_mode = INTERVAL::MODE_LOCKED;
	    else $this->end_mode = INTERVAL::MODE_UNLOCKED;
	    $this->all_mode = INTERVAL::MODE_UNLOCKED;
	}
    } else {
	$this->all_mode = INTERVAL::MODE_UNLOCKED;
	$this->end_mode = INTERVAL::MODE_UNLOCKED;
	
	$this->interval_start = 0;
	$this->interval_end = 0;
    }

    if (($props)&&(isset($props['window']))) {
	if (preg_match("/^(\d+)$/", $props['window'], $m)) {
	    $this->window_size = $m[1];
	    if ($this->interval_end) {
		if ($this->window_size) {
		    $this->window_start = dsMathPreciseSubstract($this->interval_end, $this->window_size);
		
		    if ($this->interval_start > $this->window_start) {
			// Invalid values
			$this->window_start = 0;
			$this->window_size = 0;
		    }
		} else {
		    $this->window_start = $this->interval_start;
		    $this->window_size = dsMathPreciseSubstract($this->interval_end, $this->interval_start);
		}
	    } else {
		$this->window_start = 0;
	    }
	    
	    if (($m[1])&&($this->all_mode)) $this->all_mode = INTERVAL::MODE_WINDOWLOCKED;
	
	    $this->y_min = 0;
	    $this->y_max = 0;
	} elseif (preg_match("/^(([\d.]+)-([\d.]+))?,?(([+-.eE\d]+):([+-.eE\d]+))?$/", $props['window'], $m)) {
	    if ($m[1]) {
		$this->window_start = $m[2];
		if (!$this->window_start) $this->window_start = $this->interval_start;
		$this->window_size = dsMathPreciseSubstract($m[3], $this->window_start);

		if (($this->window_start<$this->interval_start)||($this->window_size<0)) {
			// Invalid values
		    $this->window_start = 0;
		    $this->window_size = 0;
		}
		
		    // interval_start is 0 if still unlocked, so don't care
		if (($this->window_start)&&($this->end_mode)) $this->end_mode = INTERVAL::MODE_WINDOWLOCKED;
		if (($this->window_size)&&($this->all_mode)) $this->all_mode = INTERVAL::MODE_WINDOWLOCKED;
	    }
	    
	    if ($m[4]) {
		$this->y_min = $m[5];
		$this->y_max = $m[6];
		if ($this->y_min > $this->y_max) {
		    $this->y_min = 0;
		    $this->y_max = 0;
		}
	    }
	} else {
		// Invalid values
	    $this->window_size = 0;
	    $this->window_start = 0;
	    
	    $this->y_min = 0;
	    $this->y_max = 0;
	}
    } else {
	if ($this->interval_end) {
	    $this->window_start = $this->interval_start;
	    $this->window_size = dsMathPreciseSubstract($this->interval_end, $this->interval_start);
	} else {
	    $this->window_size = 0;
	    $this->window_start = 0;
	}
	
	$this->y_min = 0;
	$this->y_max = 0;
    }
    

    $this->interval_items = -1;
    $this->window_items = -1;
    
    
    $this->sequence = INTERVAL::SEQUENCE_AUTO;

    if (($reader)&&($grp)) $this->ApplyReader($reader, $grp, $flags);
    
    $this->flexibility = $flags&(INTERVAL::FLAG_FLEXIBLE|INTERVAL::FLAG_REDUCIBLE);
 }
 
 function SetupInterval($from, $to) {
    $this->interval_start = $from;
    $this->interval_end = $to;

    if ($to) {
        if ($from) {
	    $this->all_mode = INTERVAL::MODE_LOCKED;
	    $this->end_mode = INTERVAL::MODE_LOCKED;
	} else {
	    throw new ADEIException(translate("Invalid INTERVAL '%s': have end but no start", "$from-$to"));
	}
    } else {
	if ($from) $this->end_mode = INTERVAL::MODE_LOCKED;
	else $this->end_mode = INTERVAL::MODE_UNLOCKED;
	$this->all_mode = INTERVAL::MODE_UNLOCKED;
    }

    $this->window_start = 0;
    $this->window_size = 0;
 }
 
 function SetupWindow($type = INTERVAL::WINDOW_MAXIMUM, $center = 0, $size = 0, $flags = 0) {
    switch ($type) {
	case INTERVAL::WINDOW_RIGHT:
	    $this->window_start = $center;
	    $this->window_size = $size;
	    
	    if (($this->window_start)&&($this->end_mode)) $this->end_mode = INTERVAL::MODE_WINDOWLOCKED;
	    if (($this->window_size)&&($this->all_mode)) $this->all_mode = INTERVAL::MODE_WINDOWLOCKED;
	    
	    $this->sequence = INTERVAL::SEQUENCE_FORWARD;
	break;
	case INTERVAL::WINDOW_LEFT:
	    if ($size) {
		if ($center) {
		    $this->window_start = dsMathPreciseSubstract($center, $size);
		    if ($this->end_mode) $this->end_mode = INTERVAL::MODE_WINDOWLOCKED;
		} else {
		    $this->window_start = 0;
		}
		if ($this->all_mode) $this->all_mode = INTERVAL::MODE_WINDOWLOCKED;
	    } else {
		$this->window_start = $center;

		if (($center)&&($this->end_mode)) $this->end_mode = INTERVAL::MODE_WINDOWLOCKED;
	    }
	    $this->window_size = $size;
	    $this->sequence = INTERVAL::SEQUENCE_BACKWARD;
	break;
	case INTERVAL::WINDOW_MAXIMUM:
	    if (($this->interval_end)&&($this->interval_start)) {
	        $this->window_start = $this->interval_start;
		$this->window_size = dsMathPreciseSubstract($this->interval_end, $this->interval_start);
	    } elseif ($this->interval_start) {
	        $this->window_start = $this->interval_start;
		$this->window_size = 0;
	    } elseif (!$this->interval_end) {
	        $this->window_start = 0;
		$this->window_size = 0;
	    } else {
		throw new ADEIException(translate("Invalid INTERVAL: have end but no start"));
	    }

	    $this->sequence = INTERVAL::SEQUENCE_FORWARD;
	return; /* Updating is not needed */
	case INTERVAL::WINDOW_NEIGHBOURHOOD:
	    if ($size) {
		if ($center) {
		    $this->window_start = dsMathPreciseSubstract($center, $size / 2);
		    if ($this->end_mode) $this->end_mode = INTERVAL::MODE_WINDOWLOCKED;
		} else {
		    $this->window_start = 0;
		}
		$this->window_size = $size;
		if ($this->all_mode) $this->all_mode = INTERVAL::MODE_WINDOWLOCKED;
	    } else {
		$this->window_start = $center;
		$this->window_size = $size;

		if (($center)&&($this->end_mode)) $this->end_mode = INTERVAL::MODE_WINDOWLOCKED;
	    }
	    $this->sequence = INTERVAL::SEQUENCE_AUTO;
	break;
	default:
	    throw new ADEIException(translate("Invalid window type '%s' is specified", $type));
    }
    
    $this->UpdateWindow(true);
 }

 function SetSequence($seq = false) {
    if ($seq === false) $this->sequence = INTERVAL::SEQUENCE_AUTO;
    else $this->sequence = $seq;
 }

 function EnableFlexibility($reducible = false) {
    if ($reducible) {
	$this->flexibility = (INTERVAL::FLAG_FLEXIBLE|INTERVAL::FLAG_REDUCIBLE);
    } else {
	$this->flexibility = INTERVAL::FLAG_FLEXIBLE;
    }
 }

 function UpdateWindow($forbid_empty_window = false) {
    if (($this->interval_start)&&($this->interval_start > $this->window_start)) {
	$this->window_size -= dsMathPreciseSubstract($this->interval_start, $this->window_start);
	
	if ($this->window_size < 0) {
	    $this->window_start = 0;
	    $this->window_size = 0;	

	    if ($forbid_empty_window)
		throw new ADEIException(translate("The WINDOW is out of the INTERVAL '%s' range", $this->interval_start . "-" . $this->interval_end));
	} else {
	    $this->window_start = $this->interval_start;
	}
	
	$this->window_items = -1;
    }
    
    if (($this->interval_end)&&(($this->window_start + $this->window_size) > $this->interval_end)) {
	if ($this->interval_end <= $this->window_start) {
	    $this->window_start = 0;
	    $this->window_size = 0;
	    
	    if ($forbid_empty_window)
		throw new ADEIException(translate("The WINDOW is out of the INTERVAL '%s' range", $this->interval_start . "-" . $this->interval_end));
	} else {
	    $this->window_size = dsMathPreciseSubstract($this->interval_end, $this->window_start);
	}
	$this->window_items = -1;
    }
 }

 function Apply($first, $last, $records = false) {
    $end_mode = 0;
    
    if ($records !== false) $this->interval_items = $records;

    if ($this->end_mode) {
	$this->interval_start = $first;
	$this->interval_end = $last;
    } elseif ($this->all_mode) {
	$this->interval_end = $last;
    }	

    if ($this->end_mode == INTERVAL::MODE_UNLOCKED) {
	if ($this->all_mode == INTERVAL::MODE_UNLOCKED) {
	    $this->window_start = $this->interval_start;
	    $this->window_size = dsMathPreciseSubstract($this->interval_end, $this->interval_start);
	} else {
	    $this->window_start = dsMathPreciseSubstract($this->interval_end, $this->window_size);
	    $this->UpdateWindow();
	}
    } else {
	if ($this->all_mode == INTERVAL::MODE_UNLOCKED) {
	    $this->window_size = dsMathPreciseSubstract($this->interval_end, $this->window_start);
	    $this->UpdateWindow();
	} else {
	    $this->UpdateWindow();
	}
    }
 }
 
 function ApplyReader(READER &$reader, LOGGROUP &$grp, $flags = 0) {
    if (($flags&INTERVAL::NEED_AMOUNT)||($this->end_mode)||($this->all_mode)) {
	if (flags&INTERVAL::NEED_AMOUNT) {
	    $info = $reader->GetGroupInfo($grp, READER::NEED_INFO|READER::NEED_COUNT);
	    $this->Apply($info['first'], $info['last'], $info['records']);
	} else {
	    $info = $reader->GetGroupInfo($grp, READER::NEED_INFO);
	    $this->Apply($info['first'], $info['last']);
	}
    } else {
        $this->UpdateWindow();
    }
 }
 
 function ApplyCache(CACHE &$cache) {
    if (($flags&INTERVAL::NEED_AMOUNT)||($this->end_mode)||($this->all_mode)) {
	if (flags&INTERVAL::NEED_AMOUNT) {
	    $info = $cache->GetInfo(CACHE::NEED_COUNT);
	    $this->Apply($info['first'], $info['last'], $info['records']);
	} else {
	    $info = $cache->GetInfo();
	    $this->Apply($info['first'], $info['last']);
	}
    } else {
        $this->UpdateWindow();
    }
 }
 
 function Limit($min = false, $max = false) {
    if (($min)&&($min > $this->interval_start)) {
	if ($min > $this->interval_end)
	    $this->interval_start = $this->interval_end;
	else
	    $this->interval_start = $min;

	$update = 1;
    }
    if (($max)&&($max < $this->interval_end)) {
	if ($max < $this->interval_start)
	    $this->interval_end = $this->interval_start;
	else
	    $this->interval_end = $max;

	$update = 1;
    }

    if ($max) {
	$this->all_mode = INTERVAL::MODE_LOCKED;
	$this->end_mode = INTERVAL::MODE_LOCKED;
    } elseif ($min) $this->end_mode = INTERVAL::MODE_LOCKED;

    if ($update) {
	$this->interval_items = -1;
	$this->UpdateWindow();
    }
 }
 
 function GetSQL(CACHEDB &$cache, $limit = 0, $use_subseconds = false, $sequence = false) {
    $res = array();
    
    if ($limit)
	$res['limit'] = " LIMIT " . abs($limit);
    else
	$res['limit'] = "";

    if ($sequence === false) $sequence = $this->sequence;
    
    if ($use_subseconds) {
        $res['list'] = "UNIX_TIMESTAMP(time) AS timestamp, `ns`";

	if (($sequence == INTERVAL::SEQUENCE_UNSORTED)/*||($limit == 1)*/)
	    $res['sort'] = "";
	elseif (($limit < 0)||($sequence == INTERVAL::SEQUENCE_BACKWARD))
	    $res['sort'] = "ORDER BY `time` DESC, `ns` DESC";
	else
	    $res['sort'] = "ORDER BY `time` ASC, `ns` ASC";
    } else {
        $res['list'] = "UNIX_TIMESTAMP(time) AS timestamp";

	if (($sequence == INTERVAL::SEQUENCE_UNSORTED)/*||($limit == 1)*/)
	    $res['sort'] = "";
	elseif (($limit < 0)||($sequence == INTERVAL::SEQUENCE_BACKWARD))
	    $res['sort'] = "ORDER BY `time` DESC";
	else 
	    $res['sort'] = "ORDER BY `time` ASC";
    }


    $from = $this->window_start;
    if ($this->window_size > 0) {
	$to = dsMathPreciseAdd($this->window_start, $this->window_size);

	$ifrom = (int)floor($from);
	$ito = (int)floor($to);

	if (($use_subseconds)&&(($ifrom != $from)||($ito != $to))) {
	    if ($ifrom == $ito) {
		$sqlfrom = $cache->SQLTime($ifrom);
		if ($from == $ifrom) $nfrom = 0;
		else $nfrom = round(1000000000*("0" . strstr($from, ".")));
		if ($to == $ito) $nto = 0;
		else $nto = round(1000000000*("0" . strstr($to, ".")));

	        $res['cond'] = "WHERE ((`time` = $sqlfrom) AND (`ns` >= $nfrom) AND (`ns` < $nto))";
	    } else {
		$cond = "";
	        if ($ifrom != $from) {
		    $nfrom = round(1000000000*("0" . strstr($from, ".")));
		    $sqlfrom = $cache->SQLTime($ifrom);
		    $cond = "((`time` = $sqlfrom) AND (`ns` >= $nfrom))";
		    $ifrom++;
		}

		if ($ifrom != $ito) {
		    if ($cond) $cond .= " OR ";

		    $sqlfrom = $cache->SQLTime($ifrom);
	    	    $sqlto = $cache->SQLTime($ito);
    		    $cond .= "((`time` >= $sqlfrom) AND (`time` < $sqlto))";
		} else $sqlto = false;

		if ($ito != $to) {
		    if ($cond) $cond .= " OR ";
		    if (!$sqlto) $sqlto = $cache->SQLTime($ito);
		
		    $nto = round(1000000000*("0" . strstr($to, ".")));
		    $cond .= "((`time` = $sqlto) AND (`ns` < $nto))";
		}
	
		$res['cond'] = "WHERE ($cond)";
	    }
	} else {
	    $sqlfrom = $cache->SQLTime($ifrom);
	    $sqlto = $cache->SQLTime($ito);
    	    $res['cond'] = "WHERE ((`time` >= $sqlfrom) AND (`time` < $sqlto))";
	}
    } else {
	if ($this->sequence == INTERVAL::SEQUENCE_BACKWARD) $sign = '<';
	else $sign = '>';

	$ifrom = (int)floor($from);
	$sqlfrom = $cache->SQLTime($ifrom);

	if (($use_subseconds)&&($ifrom != $from)) {
	    $nfrom = round(1000000000*("0" . strstr($from, ".")));
	    $res['cond'] = "WHERE (((`time` = $sqlfrom) AND (`ns` $sign $nfrom)) OR (`time` $sign $sqlfrom))";
	} else {
	    $res['cond'] = "WHERE `time` $sign $sqlfrom";
	}
    }
    
    return $res;
 }
 
 function GetWindowStart() {
//    if ($this->window_start)||
	return $this->window_start;
 }

 function GetWindowEnd() {
    return dsMathPreciseAdd($this->window_start, $this->window_size);
 }
 
 function GetWindowSize() {
    return $this->window_size;
 }
 
 function IsEmpty() {
    if (!$this->window_start) return true;
    return false;
 }
 
 function GetIntervalStart() {
    return $this->interval_start;
 }
 
 function GetIntervalEnd() {
    return $this->interval_end;
 }
 
 function GetName($format = NAME_FORMAT_DEFAULT) {
    switch ($format) {
	case NAME_FORMAT_HUMAN:
	    throw new ADEIException(translate("Not implemented yet"));
	break;
	case NAME_FORMAT_ISO8601:
	    if (($this->window_start)&&($this->window_size))
	        return date("Ymd\THis", floor($this->window_start)) . "-" . date("Ymd\THis", ceil($this->window_start + $this->window_size));
	    else if ($this->window_size) {
		$ws = $this->window_size;
		$msg = "";
		if ($ws >= 1) {
		    if ($ws > 86400) {
			$msg .= floor($ws / 86400) . "d";
			$ws = $ws % 86400;
		    }
		    if ($ws > 3600) {
			$msg .= floor($ws / 3600) . "h";
			$ws = $ws % 3600;
		    }
		    if ($ws > 60) {
			$msg .= floor($ws / 60) . "m";
			$ws = $ws % 60;
		    }
		    $msg .= floor($ws) . "s";
		} else if ($ws >= 0.001) {
		    $msg = floor($ws*1000) . "ms";
		} else if ($ws >= 0.000001) {
		    $msg = floor($ws*1000000) . "us";
		} else {
		    $msg = floor($ws*1000000000) . "ns";
		}
	    } else {
		return "everything";
	    }
	break;
	default:
	    throw new ADEIException(translate("Unknown format is requested"));
    }
 }
}

?>