<?php

class CACHEReader extends READER {
 var $cache;

 function __construct(&$props, CACHEDB &$cache = NULL) {
    parent::__construct($props);
    if ($cache) $this->cache = &$cache;
    else {
	if ($props instanceof SOURCERequest)
	    $this->cache = new CACHEDB($props);
	else
	    $this->cache = NULL;
    }
 }
 
 function GetGroupInfo(LOGGROUP &$grp = NULL, $flags = 0) {
    $groups = array();

    $res = $this->cache->ListCachedGroups();
    foreach ($res as $gid) {
	if (($grp)&&(strcmp($grp->gid, $gid))) continue;

	$groups[$gid] = array(
	    'gid' => $gid,
	    'name' => $gid
	);

	if ($flags&READER::NEED_INFO) {
	    $postfix = $this->cache->GetCachePostfix($gid);
	    
	    $flags = 0;
	    if ($flags&READER::NEED_COUNT) $flags |= CACHEDB::NEED_COUNT;
	    $info = $this->cache->GetCacheInfo($postfix, $flags);

	    if (!$info)
		throw new ADEIException(translate("The CACHE for group (%s) is empty", $grp->gid));

	    foreach ($info as $key => &$value) {
		$groups[$gid][$key] = $value;
	    }
	    
	    if ($flags&READER::NEED_ITEMINFO) {
		$groups[$gid]['items'] = $this->cache->GetCacheItemList($postfix);
	    }
	}
    }

    return $grp?$groups[$grp->gid]:$groups;
 }
 
 function GetItemList(LOGGROUP &$grp = NULL, MASK &$mask = NULL, $flags = 0) {
    $grp = $this->CheckGroup($grp);

    $postfix = $this->cache->GetCachePostfix($grp->gid);
    if (!$mask) $mask = $this->CreateMask($grp);

    return $this->cache->GetCacheItemList($postfix, $mask, 0);
 }

 function GetData(LOGGROUP &$grp = NULL, $from = 0, $to = 0) {
    $grp = $this->CheckGroup($grp);

    $postfix = $this->cache->GetCachePostfix($grp->gid);

    $ivl = $this->CreateInterval($grp);
    $ivl->Limit($from, $to);

    $minfo = array("db_mask"=>"");
    $mask = $this->cache->CreateMask($minfo);
    
    return $this->cache->GetCachePoints($postfix, $mask, $ivl, CACHEDB::TYPE_ALL);
 }

 function HaveData(LOGGROUP &$grp = NULL, $from = 0, $to = 0) {
    $grp = $this->CheckGroup($grp);

    $postfix = $this->cache->GetCachePostfix($grp->gid);

    $ivl = $this->CreateInterval($grp);
    $ivl->Limit($from, $to);

    $mask = NULL;

    $points = $this->cache->GetCachePoints($postfix, $mask, $ivl, CACHEDB::TYPE_ALL, 1);

    $points->rewind();
    return $points->valid();
 }
}

?>