<?php

class DBLogGroup extends LOGGROUP {
 var $table;
 
 function __construct(array &$info, DBReader &$rdr, $flags = 0) {
    parent::__construct($info);

    if (is_array($rdr->groups)) {
	foreach($rdr->groups as $re => &$table) {
	    if (preg_match($re, $this->gid)) {
		$this->table = preg_replace($re, $table, $this->gid);
		return;
	    }
	}
    }	
    $this->table = &$this->gid;
 }
}

class DBReader extends READER {
 var $db;
 
 var $groups;
 var $tables;
 var $columns;
 
 var $data_request;
 var $time_sort;
 
 var $monitor_timings;
 var $emit_delays;

 function __construct(&$props) {
    parent::__construct($props);
    $this->db = new DATABASE($this->server);
    
    $this->groups = $this->opts->Get('groups');
    if ($this->groups) $this->group_class = "DBLogGroup";
    
    $this->tables = $this->opts->Get('tables');
    $this->columns = $this->opts->Get('columns');
    $this->data_request = false;

    $time_format = $this->opts->Get("timeformat");
    if ($time_format) $this->time_format = $time_format;
    else $this->time_format = $this->db->GetTimeFormat();

    $time_zone = $this->opts->Get("timezone");
    if ($time_zone) $this->time_zone = new DateTimeZone($time_zone);
    
    $time_sort = $this->opts->Get("timesort");
    if ($time_sort) {
	if (is_string($time_sort)) $this->time_sort = "ORDER BY " . $time_sort;
	else if ($time_sort>0) $this->time_sort = "ORDER BY " . $this->columns['time'] . " ASC";
	else $this->time_sort = "ORDER BY " . $this->columns['time'] . " DESC";
    } else $this->time_sort = "";

    /* per db only, no per group handling here */    
    $this->emit_delays = $this->opts->Get("emit_delays");
    $this->monitor_timings = $this->opts->Get("monitor_timings");
 }
 
 
 function GetGroupInfo(LOGGROUP &$grp = NULL, $flags = 0) {
    $groups = array();

    $res = $this->db->ShowTables();
    foreach ($res as $row) {
	$gid = $row[0];
	
	if (is_array($this->tables)) {
	    $found = false;
	    foreach ($this->tables as $re => &$info) {
		if (preg_match($re, $gid)) {
		    if (is_array($info)) {
			if (isset($info['title'])) {
			    $name = preg_replace($re, $info['title'], $gid);
			    $gid = preg_replace($re, $info['gid'], $gid);
			} else {
			    $gid = preg_replace($re, $info['gid'], $gid);
			    $name = $gid;
			}
		    } else {
			$gid = preg_replace($re, $info, $gid);
			$name = $gid;
		    }
		    $found = true;
	    	    break;
		}
	    }
	    if (!$found) continue;
	    if (($grp)&&($grp->gid != $gid)) continue;
	} else if ($tables) {
	    if (($grp)&&($grp->gid != $gid)) continue;
	    if (!preg_match($this->tables, $gid)) continue;
	    $name = $gid;
	} else {
	    throw new ADEIException(translate("DBReader has no tables are configured in"));
	}

	$groups[$gid] = $row;
	$groups[$gid]['gid'] = $gid;
	$groups[$gid]['name'] = $name;

	if ($flags&READER::NEED_INFO) {
	    if ($grp) {
		$grzeus = $grp;
	    } else {
		$ginfo = array("db_group" => $gid);
		$grzeus = $this->CreateGroup($ginfo);
	    }
	    
	    $tc = $this->columns['time'];
	    $req = "MIN($tc), MAX($tc)";
	    if ($flags&READER::NEED_COUNT) 
		$req .= ", COUNT($tc)";

	    if ($this->monitor_timings) {
		$tt = $this->req->GetGroupOption("monitor_timings", $grp);
		if (is_array($tt)) {
		    if (isset($tt['query_limit'])) $tt_limit = $tt['query_limit'];
		    else $tt_limit = 1000000;
		    $tt_exception = $tt['raise_exception'];
		} else {
		    $tt_limit = 1000000;
		    $tt_exception = false;
		}
		$tt_limit += 1000000*microtime(true);
/*
	        $tod = gettimeofday();
		$tod['usec'] += $tt_limit%1000000;
		if ($tod['usec'] > 999999) {
		    $tod['usec'] -= 1000000;
		    $tod['sec']++;
		}
		$tod['sec'] += $tt_limit / 1000000;
*/
	    }
	    
	    if ($this->emit_delays) {
		log_message("Sleeping");
		usleep($this->emit_delays);
	    }
	    
	    $valres = $this->db->Query("SELECT $req FROM " . $grzeus->table);
	    $vals = $valres->fetch(PDO::FETCH_NUM);
	    $valres = NULL;
	    
	    if ($this->monitor_timings) {
		if (1000000 * microtime(true) > $tt_limit) {
		    $msg = translate("The query on group '%s' is exceeded allowed execution time (%d msec). This normally indicates inappropriate indexing of the source database. You can overcome the problem by setting '%s' and '%s' options", $grp->gid, ceil(1000*microtime(true) - $tt_limit/1000), "use_cache_reader", "fill_raw_first");
		    if ($tt_exception)
			throw new ADEIException($msg);
		    else
			log_message($this->req->GetLocationString() . ": " . $msg);
		}
	    }

	    if (($vals)&&(($vals[0])||($vals[1]))) {
	        $opts = $this->req->GetGroupOptions($grzeus);
		$limit = $opts->GetDateLimit();

	    
		$groups[$gid]['first'] = $this->ExportUnixTime($vals[0]);
	        if (($limit[0])&&($limit[0] > $groups[$gid]['first'])) {
	    	    $groups[$gid]['first'] = $limit[0];
		}
	    
		$groups[$gid]['last'] = $this->ExportUnixTime($vals[1]);
		if (($limit[1])&&($limit[1] < $groups[$gid]['last'])) {
	    	    $groups[$gid]['last'] = $limit[1];
		}

		if ($flags&READER::NEED_COUNT) 
		    $groups[$gid]['records'] = $vals[2];
	    } else {
		unset($groups[$gid]['first']);
		unset($groups[$gid]['last']);
	    }

	    if ($flags&READER::NEED_ITEMINFO) {
		$groups[$gid]['items'] = $this->GetItemList($grzeus);
	    }
	}
    }
    
    return $grp?$groups[$grp->gid]:$groups;
 }
 
 function GetItemList(LOGGROUP &$grp = NULL, MASK &$mask = NULL, $flags = 0) {
    $grp = $this->CheckGroup($grp);
    if (!$mask) $mask = $this->CreateMask($grp);

    $items = array();

    $resp = $this->db->ShowColumns($grp->table);

    $pos = 0; $rpos = 0;
    foreach ($resp as $row) {
	$name = $row[0];
	if (!preg_match($this->columns['data'], $name)) continue;

	if (!$mask->Check($pos++)) continue;
    
	$items[$rpos] = array(
	    "id" => $pos - 1,
	    "name" =>  $name,
	    "column" => $name
	);
	$rpos++;
    }
    
    if (!$name)
	throw new ADEIException(translate("DBReader can't find any column in table (%s)", $grp->table));

    if (!$pos)
	throw new ADEIException(translate("DBReader is not able to find any column matching filter (%s) in table (%s)", $this->columns['data'], $grp->table));
    
    return $items;
 }

 function FindColumns(&$grp) {
    if (!$this->data_request[$grp->gid]) {
	if ($this->time_module)
	    $time_column = $this->columns['time'];
	else
	    $time_column = $this->db->GetTimeRequest($this->columns['time']);
	    
	$data_columns = "";

	$items = $this->GetItemList($grp);
	foreach ($items as $item) {
	    $data_columns .=  "\"" . $item['column'] . "\", ";
	}
	
	$this->data_request[$grp->gid] = "$data_columns $time_column";
	
    }
 }    

 function GetData(LOGGROUP &$grp = NULL, $from = 0, $to = 0) {
    $grp = $this->CheckGroup($grp);
    if ((!$from)||(!$to)) {
        $ivl = $this->CreateInterval($grp);
	$ivl->Limit($from, $to);
	
	$from = $ivl->GetWindowStart();
	$to = $ivl->GetWindowEnd();
    }

    $this->FindColumns($grp);    


//    echo("SELECT " . $this->data_request[$grp->gid]  . " FROM " . $grp->table . " WHERE \"" . $this->columns['time'] . "\" BETWEEN '" . $this->ImportUnixTime($from) . "' and '" . $this->ImportUnixTime($to) . '\'' . "\n\n");
    if ($this->emit_delays) {
	log_message("Sleeping");
	usleep($this->emit_delays);
    }
    
    $stmt = $this->db->Prepare("SELECT " . $this->data_request[$grp->gid]  . " FROM " . $grp->table . " WHERE \"" . $this->columns['time'] . "\" BETWEEN '" . $this->ImportUnixTime($from) . "' and '" . $this->ImportUnixTime($to) . '\' ' . $this->time_sort);
    return new DATABASEData($this, $stmt);
 }

 function HaveData(LOGGROUP &$grp = NULL, $from = 0, $to = 0) {
    $grp = $this->CheckGroup($grp);
    if (!$to) $to = time();

    if ($this->emit_delays) {
	log_message("Sleeping");
	usleep($this->emit_delays);
    }
    $res = $this->db->Query($this->db->SelectRequest($grp->table, array($this->columns['time']), array(
	"condition" => "\"" . $this->columns['time'] . "\" BETWEEN '" . $this->ImportUnixTime($from) . "' and '" . $this->ImportUnixTime($to) . '\'',
	"limit" => 1)));
    if ($res->fetch(PDO::FETCH_NUM)) return true;
    return false;
 }
 
}

?>