<?php

class TESTData implements Iterator {
 var $period;
 var $from, $to;
 var $pos;
 
 var $items;

 var $start;
 
 const DEFAULT_START = "January 1, 2000";
   
 public function __construct(TESTReader &$reader, OPTIONS &$opts, &$items, INTERVAL &$ivl) {
    $period = $opts->Get('period');
    $this->period = $period;
    
    $from = $ivl->GetWindowStart();
    $ifrom = ceil($from);
    $rem = $ifrom % $period;
    if ($rem) $this->from = $ifrom + ($period - $rem);
    else $this->from = $ifrom;

    $this->to = $ivl->GetWindowEnd();
    
    $this->items = &$items;

    $limit = $opts->GetDateLimit(TESTData::DEFAULT_START, time());
    $this->start = $limit[0]; 
 }
 
 function rewind() {
    $this->pos = $this->from;
 }

 function current() {
    $x = $this->pos - $this->start;
    $period = $this->period;

    $max = getrandmax();
    
    $values = array();
    foreach ($this->items as &$item) {
	$y = 0;
/*	
	if ($item['name']=="Sinus") {
//	    echo $item['func'] . "\n";
	    echo $x . " " . ($x/$period) . "  ";
	    eval('echo deg2rad($x/$period) . "\n";');
	}
*/
	if (eval($item['func'] . ";") === false)
	    throw new ADEIException(translate("Invalid function(%s) is supplied to TESTReader", $item['func']));

	if ($item['noise']) $y += 2*$item['noise']*((rand() - $max/2)/$max);

	array_push($values, $y);
    }
    
    return $values;
 }

 function key() {
    return $this->pos;
 }

 function next() {
    $this->pos += $this->period;
 }

 function valid() {
    return ($this->pos < $this->to);
 }

}



class TESTReader extends READER {
 var $cache;

 var $items;
 var $groups;
 
 function __construct(&$props) {
    parent::__construct($props);

    $this->items = array(
	array(
	    "name" => _("Line"),
	    "func" => '$y=0',
	    "noise" => 0.1
	),
	array(
	    "name" => _("Sinus"),
	    "func" => '$y=sin(deg2rad($x/$period))'
	),
	array(
	    "name" => _("Cosinus"),
	    "func" => '$y=cos(deg2rad($x/$period))'
	)
    );
    
    $this->groups = array(
	'default' => array(
	    'name' => "Default Group"
	)
    );
 }
 
 function GetGroupInfo(LOGGROUP &$grp = NULL, $flags = 0) {
    $groups = array();

    foreach ($this->groups as $gid => &$group) {
	if (($grp)&&(strcmp($grp->gid, $gid))) continue;

	$groups[$gid] = array(
	    'gid' => $gid,
	    'name' => $group['name']
	);
    
	if ($flags&READER::NEED_INFO) {
	    if ($grp) {
		$grtest = $grp;
		$opts = $this->opts;
	    } else {
		$ginfo = array("db_group" => $gid);
		$grtest = $this->CreateGroup($ginfo);
		$opts = $this->req->GetGroupOptions($grtest);
	    }
	    
	    $limit = $opts->GetDateLimit(TESTData::DEFAULT_START, time());
	    $groups[$gid]['first'] = $limit[0]; 
	    $groups[$gid]['last'] = $limit[1]; 

	    if ($flags&READER::NEED_COUNT) {
	    	$period = $opts->Get('period');
		$groups[$gid]['records'] = ($groups[$gid]['last'] - $groups[$gid]['first']) / $period;
	    }

	    if ($flags&READER::NEED_ITEMINFO) {
		$groups[$gid]['items'] = $this->GetItemList($grtest);
	    }
	}
    }

    return $grp?$groups[$grp->gid]:$groups;
 }
 
 function GetItemList(LOGGROUP &$grp = NULL, MASK &$mask = NULL, $flags = 0) {
    $grp = $this->CheckGroup($grp);
    if (!$mask) $mask = $this->CreateMask($grp);

    $items = array();

    $opts = $this->req->GetGroupOptions($grtest);
    $items = $opts->Get('items', $this->items);

    $res = array();

    for ($i = 0, $pos = 0; isset($items[$i]); $i++) {
	if (!$mask->Check($i)) continue;

	$res[$pos++] = array(
	    "id" => $i,
	    "name" =>  $items[$i]['name']
	);
    }
    
    return $res;
 }

 function GetData(LOGGROUP &$grp = NULL, $from = 0, $to = 0) {
    $grp = $this->CheckGroup($grp);

    $ivl = $this->CreateInterval($grp);
    $ivl->Limit($from, $to);

    $opts = $this->req->GetGroupOptions($grtest);
    $items = $opts->Get('items', $this->items);
    
    return new TESTData($this, $this->req->GetGroupOptions($grp), $items, $ivl);
 }

 function HaveData(LOGGROUP &$grp = NULL, $from = 0, $to = 0) {
    $grp = $this->CheckGroup($grp);

    $ivl = $this->CreateInterval($grp);
    $ivl->Limit($from, $to);
    
    $period = $this->req->GetGroupOption('period', $grp);

    $from = $ivl->GetWindowStart();
    $to = $ivl->GetWindowEnd();
    
    $period = $this->req->GetGroupOption('period', $grp);
    if (($from - $to) > 2 * $period) return true;
    else {
	$ifrom = ceil($from);
	$rem = $ifrom % $period;
	if ($rem) $curfrom = $ifrom + ($period - $rem);
	else $curfrom = $ifrom;
	
	if ($curfrom < $to) return true;
    }
    
    return false;
 }
}

?>